require "rack"
require "forwardable"
require "securerandom"

require_relative "version"
require_relative "extends"

require_relative "ext/convenience"

require_relative "routing/eval"
require_relative "routing/methods"


module Pretzel
  class Core
    extend Pretzel::Extension
    include Pretzel::Routing::Evaluation
    extend Pretzel::Routing::Methods

    extends Pretzel::Extension::Convenience

    attr_reader :request, :response, :parameters

    class << self
      extend Forwardable
      def_delegators :builder, :map, :use

      def token
        @token ||= SecureRandom.hex(32)
      end

      def routes
        @routes ||= Hash.new { |hash, key| hash[key] = [] }
      end

      def filters
        @filters ||= Hash.new
      end

      alias_method :new!, :new
      def new(*args, &block)
        builder.run new!(*args, &block)
        builder
      end

      def builder
        @builder ||= Rack::Builder.new
      end
    end # end self

    def initialize
      self.class.use Rack::Session::Cookie, secret: self.class.token
      self.class.use Rack::Static, :urls => ["/assets"]
    end

    def call(env)
      dup.call!(env)
    end

    def call!(env)
      @request = Rack::Request.new(env)
      @response = Rack::Response.new
      @parameters = request.params

      @response["Server"] = "Ruby/#{RUBY_VERSION}"
      @response["X-Generated-By"] = "Pretzel/#{Pretzel::VERSION}"

      evaluate_route

      @response.finish
    end

    def halt(*res)
      response.status = res.detect{|x| x.is_a?(Fixnum) } || 200
      response.header.merge!(res.detect{|x| x.is_a?(Hash) } || {})

      if self.class.routes[response.status].empty?
        response.body = [res.detect{|x| x.is_a?(String) } || ""]
      else
        self.class.routes[response.status].each do |route, block|
          response.write instance_eval(&block)
        end
      end

      throw :halt, response
    end    
  end # end class
end # end module