module Pretzel
  module Routing
    module Evaluation

      def self.included(parent)
        parent.extend ClassMethods
      end

      # For all regex lovers.
      module ClassMethods
        def compile_route(pattern)
          keys = []
          pattern = pattern[0...-1] if pattern.end_with?("/")
          pattern.gsub!(/(:\w+)/) do # Get all colon-prefixed params
            keys << $1[1..-1] # Put the name of them into the keys array.
            "([^/?#]+)" # Substitute with a capture group.
          end
          # Return the regex and the param names.
          if pattern != "/" # No trailing slash for the root route.
            [%r{^#{pattern}\/?$}, keys] 
          else
            [%r{^#{pattern}$}, keys] 
          end
        end
      end

      private
      def evaluate_route
        catch(:halt) do
          method = request.request_method.downcase.to_sym

          self.class.routes[method].each do |route, block|
            if match = request.path_info.match(route[0])
              if (captures = match.captures) && !captures.empty?
                url_params = Hash[*route[1].zip(captures).flatten]
                @parameters = url_params.merge(parameters)
              end

              self.class.filters[:before].each { |filter| instance_exec(&filter) } if self.class.filters[:before]
              response.write instance_eval(&block)
              self.class.filters[:after].each { |filter| instance_exec(&filter) } if self.class.filters[:after]
              
              return
            end
          end
          
          halt 404
        end
      end

    end
  end
end